@@ -16,13 +16,14 @@ digitalocean_provider = sys.modules[__name__]
 SSH_KEY_PATH = os.path.expanduser('~/.ssh/id_rsa')
 
 
-@lru_cache(maxsize=1)
-def _digitalocean_ssh_key_fingerprint():
-    p = run('ssh-keygen -E md5 -lf ~/.ssh/id_rsa.pub')
+@lru_cache()
+def _digitalocean_ssh_key_fingerprint(username, ssh_key_path):
+    p = run(f'ssh-keygen -E md5 -lf {ssh_key_path}.pub')
     fingerprint = p.stdout.split(' ')[1][4:]
     p = run(f'doctl compute ssh-key get {fingerprint}')
     if p.returncode != 0:
-        p = run(f'doctl compute ssh-key import python-rc ~/.ssh/id_rsa.pub')
+        p = run(
+            f'doctl compute ssh-key import username --public-key-file {ssh_key_path}.pub')
     return fingerprint
 
 
@@ -45,7 +46,11 @@ def _exist(id_):
     return p.returncode == 0
 
 
-def get(name):
+def get(name, username=None, ssh_key_path=None, **kwargs):
+    if username is None:
+        username = 'root'
+    if ssh_key_path is None:
+        ssh_key_path = SSH_KEY_PATH
     p = run(
         f'doctl compute droplet list --no-header --format Region,Name,PublicIPv4,ID')
     lines = p.stdout.strip('\n').split('\n')
@@ -53,7 +58,7 @@ def get(name):
         zone, name_, ip, id_ = re.split(r'\s+', line)
         if name_ == name:
             m = Machine(provider=digitalocean_provider, name=name,
-                        zone=zone, ip=ip, username='root', ssh_key_path=SSH_KEY_PATH)
+                        zone=zone, ip=ip, username=username, ssh_key_path=ssh_key_path)
             m.id = id_
             return m
     return None
@@ -78,7 +83,7 @@ def shutdown(machine):
         raise MachineShutdownException(p.stderr)
 
 
-def create(name, *, image, region, size, firewall_names=None):
+def create(name, *, image, region, size, firewall_name=None, username=None, ssh_key_path=None):
     """
     Available images:
     user images: doctl compute snapshot list
@@ -97,15 +102,26 @@ def create(name, *, image, region, size, firewall_names=None):
     machine = get(name)
     if machine:
         raise MachineCreationException(f'Machine {name} is already exist')
-    cmd = f'doctl compute droplet create {name} --region {region} --size {size} --image {image} --ssh-keys {_digitalocean_ssh_key_fingerprint()}'
-    if firewall_names:
-        cmd += ' --tag-names ' + ','.join(firewall_names)
+    if username is None:
+        username = 'root'
+    if ssh_key_path is None:
+        ssh_key_path = SSH_KEY_PATH
+    cmd = f'doctl compute droplet create {name} --region {region} --size {size} --image {image} --ssh-keys {_digitalocean_ssh_key_fingerprint(username, ssh_key_path)}'
+    if firewall_name:
+        cmd += ' --tag-name ' + firewall_name
     cmd += ' --wait'
     p = run(cmd)
     if p.returncode != 0:
         raise MachineCreationException(p.stderr)
-    machine = get(name)
+    machine = get(name, ssh_key_path=ssh_key_path)
     machine.wait_ssh()
+    if username != 'root':
+        with open(os.path.expanduser(f'{ssh_key_path}.pub')) as f:
+            pubkey = f.read().strip()
+        p = machine.ensure_user(username, pubkey=pubkey)
+        if p.returncode != 0:
+            raise MachineCreationException(p.stderr)
+    machine.username = username
     return machine
 
 
@@ -147,7 +163,7 @@ def delete(machine):
 
 
 def create_firewall(name, *, direction='in', ports, ips=['0.0.0.0/0']):
-    cmd = f'doctl compute firewall create {name} --tag-names {name}'
+    cmd = f'doctl compute firewall create --name {name} --tag-names {name}'
     rules = []
     for port in ports:
         if port == 'icmp':
@@ -155,7 +171,7 @@ def create_firewall(name, *, direction='in', ports, ips=['0.0.0.0/0']):
             rule += ',address:'.join(ips)
         else:
             protocol, port = port.split(':')
-            rule = f'protocol:{protocol},port:{port},address:'
+            rule = f'protocol:{protocol},ports:{port},address:'
             rule += ',address:'.join(ips)
         rules.append(rule)
     rules = ' '.join(rules)
@@ -166,7 +182,18 @@ def create_firewall(name, *, direction='in', ports, ips=['0.0.0.0/0']):
     else:
         raise FirewallRuleCreationException(
             'direction must be either "in" or "out"')
+    print(cmd)
     p = run(cmd)
     if p.returncode != 0:
         raise FirewallRuleCreationException(p.stderr)
-    return Firewall(name, provider=digitalocean_provider, direction=direction, action=action, ports=ports, ips=ips)
+    return Firewall(name, provider=digitalocean_provider, direction=direction, action='allow', ports=ports, ips=ips)
+
+
+def get_image(name):
+    p = run(f'doctl compute image list')
+    images = p.stdout.split('\n')
+    for i in images:
+        id_, n, *_ = re.split(r'\s+', i)
+        if n == name:
+            return id_
+    return None
